#!/usr/bin/env python
'''
This file contains Python command example for setting and reading atcam counters.
It can be used as a debugging utility for reading the atcam & erp counters.
'''
import sys
import errno
import pdb
import time
from python_sdk_api.sx_api import *
from python_sdk_api.sxd_api import *
from test_infra_common import *
import argparse

######################################################
#    defines
######################################################
SWID = 0
DEVICE_ID = 1

ERR_FILE_LOCATION = '/tmp/python_err_log.txt'

parser = argparse.ArgumentParser(description='generate an extended debug dump',
                                 formatter_class=argparse.ArgumentDefaultsHelpFormatter)
parser.add_argument('--device_id', default=1, type=lambda x: int(x, 0), help='The device id to extract the FW info from')
parser.add_argument('--path', default=None, help='a path where the dump will be saved')
parser.add_argument('--force', default=False, action='store_true', help='forces a refrash of the debug instructions DB')
args = parser.parse_args()


def main():

    print "[+] opening sdk"
    rc, handle = sx_api_open(None)
    print ("sx_api_open handle:0x%x , rc %d " % (handle, rc))
    if (rc != SX_STATUS_SUCCESS):
        print "Failed to open api handle.\nPlease check that SDK is running."
        sys.exit(errno.EACCES)

    dbg_params_p = new_sx_dbg_extra_info_t_p()
    dbg_params_p.dev_id = args.device_id
    dbg_params_p.path = args.path
    dbg_params_p.timeout_usec = 0  # not yet supported
    dbg_params_p.is_async = False  # not yet supported
    dbg_params_p.force_db_refresh = args.force
    rc = sx_api_dbg_generate_dump_extra(handle, dbg_params_p)
    delete_sx_dbg_extra_info_t_p(dbg_params_p)
    assert rc == SX_STATUS_SUCCESS, "Failed to generate extended debug dump for device:{0}".format(dbg_params_p.dev_id)

    print("Success.")

    sx_api_close(handle)


################################################################################
#                             Main                                             #
################################################################################
if __name__ == "__main__":
    main()
