#!/usr/bin/env python
"""
This file contains a python commands example for the LAG module.
Python commands syntax is very similar to the SwitchX SDK APIs.
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.
"""

import sys
import errno
from python_sdk_api.sx_api import *
from python_sdk_api.sxd_api import *
from test_infra_common import *
import argparse

######################################################
#    defines
######################################################
SWID = 0
DEVICE_ID = 1
MAX_PORTS = 64


def auto_int(x):
    return int(x, 0)


ERR_FILE_LOCATION = '/tmp/python_err_log.txt'

if len(sys.argv) < 3:
    print "Bad parameters."
    print "    " + sys.argv[0] + " --cmd <add(1)|create(12)|delete(3)|destroy(13)> [--lag_id 0x10000000]" \
        + "--ports 0x10100 0x10300"
    sys.exit(0)

parser = argparse.ArgumentParser(description='sx_api_lag_port_group_set API')
parser.add_argument('--cmd', type=str, choices=["ADD", "add", "1", "DELETE", "delete", "3", "CREATE", "create",
                                                "12", "DESTROY", "destroy", "13"],
                    help='Command: ADD|SET|DELETE|DELETE_ALL')
parser.add_argument('--lag_id', type=auto_int, default=0, help='Egress port ')
parser.add_argument('--ports', type=auto_int, nargs='+', help='Ports')
args = parser.parse_args()

cmd_dict = {
    "ADD": SX_ACCESS_CMD_ADD,
    "DESTROY": SX_ACCESS_CMD_DESTROY,
    "DELETE": SX_ACCESS_CMD_DELETE,
    "CREATE": SX_ACCESS_CMD_CREATE,
    "add": SX_ACCESS_CMD_ADD,
    "destroy": SX_ACCESS_CMD_DESTROY,
    "delete": SX_ACCESS_CMD_DELETE,
    "create": SX_ACCESS_CMD_CREATE,
    "1": SX_ACCESS_CMD_ADD,
    "13": SX_ACCESS_CMD_DESTROY,
    "3": SX_ACCESS_CMD_DELETE,
    "12": SX_ACCESS_CMD_CREATE,
}


print "Cmd: %s (%d)" % (args.cmd, cmd_dict[args.cmd])
print "Lag id: 0x%x" % args.lag_id
if args.ports:
    for i, port in enumerate(args.ports):
        print ("Ports[%d]: 0x%x" % (i, port))

# file_exist = os.path.isfile(ERR_FILE_LOCATION)
# sys.stderr = open(ERR_FILE_LOCATION, 'w')
# if not file_exist:
#     os.chmod(ERR_FILE_LOCATION, 0o777)

######################################################
#    functions
######################################################


def main():
    #     pdb.set_trace()
    print "[+] opening sdk"
    rc, handle = sx_api_open(None)
    print ("sx_api_open handle:0x%x , rc %d " % (handle, rc))
    if (rc != SX_STATUS_SUCCESS):
        print "Failed to open api handle.\nPlease check that SDK is running."
        sys.exit(errno.EACCES)

    ports_cnt = 0
    if args.ports:
        ports_cnt = len(args.ports)
    ports = new_sx_port_log_id_t_arr(MAX_PORTS)
    for i in range(MAX_PORTS):
        sx_port_log_id_t_arr_setitem(ports, i, 0)
    if args.ports:
        for i, port in enumerate(args.ports):
            sx_port_log_id_t_arr_setitem(ports, i, port)

    lag_id_p = new_sx_port_log_id_t_p()
    sx_port_log_id_t_p_assign(lag_id_p, args.lag_id)

    rc = sx_api_lag_port_group_set(handle, cmd_dict[args.cmd], SWID, lag_id_p, ports, ports_cnt)
    assert rc == SX_STATUS_SUCCESS, "sx_api_lag_port_group_set failed. [%s(%d)]" % (sx_status_dict[rc], rc)

    lag_id = sx_port_log_id_t_p_value(lag_id_p)

    print "SET results"
    print "    LAG ID:0x%x " % lag_id

    ports_cnt_p = new_sx_port_log_id_t_p()
    ports_cnt = MAX_PORTS
    sx_port_log_id_t_p_assign(ports_cnt_p, ports_cnt)
    for i in range(ports_cnt):
        sx_port_log_id_t_arr_setitem(ports, i, 0)

    rc = sx_api_lag_port_group_get(handle, SWID, lag_id, ports, ports_cnt_p)

    if cmd_dict[args.cmd] == SX_ACCESS_CMD_DESTROY:
        assert rc == SX_STATUS_ENTRY_NOT_FOUND, "sx_api_lag_port_group_get after \"Destroy command\" returns" + \
                                                " [%s(%d)]" % (sx_status_dict[rc], rc)
    else:
        assert rc == SX_STATUS_SUCCESS, "sx_api_lag_port_group_get failed. [%s(%d)]" % (sx_status_dict[rc], rc)

    print "GET results"
    print "    LAG ID:0x%x " % lag_id

    if cmd_dict[args.cmd] != SX_ACCESS_CMD_DESTROY:
        ports_cnt = sx_port_log_id_t_p_value(ports_cnt_p)
        print "    Ports cnt:%d " % ports_cnt
        if ports_cnt > 0:
            for i in range(0, ports_cnt):
                port = sx_port_log_id_t_arr_getitem(ports, i)
                print ("Ports[%d]: 0x%x" % (i, port))

    rc = sx_api_close(handle)


################################################################################
#                             Main                                             #
################################################################################
if __name__ == "__main__":
    main()
