#!/usr/bin/env python
'''
This file contains Python command example for the PORT module.
Python commands syntax is very similar to the Switch SDK APIs.
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below demonstrates the configuration of port CRC attributes.
'''
import sys
import errno
from python_sdk_api.sx_api import *

SPECTRUM_SWID = 0
PORT1 = 0x10100
PORT2 = 0x10300
PRIO = 6

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x, rc %d " % (handle, rc))
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)


log_port = PORT1
crc_params_p = new_sx_port_crc_params_t_p()
rc = sx_api_port_crc_params_get(handle, log_port, crc_params_p)
crc_params = sx_port_crc_params_t_p_value(crc_params_p)
print("sx_api_port_crc_params_get [log_port=0x%x, ingress=%d, egress=%d, rc=%d]" %
      (log_port, crc_params.bad_crc_ingress_mode, crc_params.crc_egress_recalc_mode, rc))
if rc != 0:
    sys.exit(errno.EACCES)


print("Set CRC Params to allow bad CRCs and prevent CRC recalculation")
crc_params = sx_port_crc_params_t()
crc_params.bad_crc_ingress_mode = SX_PORT_BAD_CRC_INGRESS_MODE_FORWARD
crc_params.crc_egress_recalc_mode = SX_PORT_CRC_EGRESS_RECALC_MODE_PREVENT
sx_port_crc_params_t_p_assign(crc_params_p, crc_params)
rc = sx_api_port_crc_params_set(handle, log_port, crc_params_p)
print("sx_api_port_crc_params_set return %d" % (rc))
if rc != 0:
    sys.exit(errno.EACCES)

print("Set CRC Params to Default with sx_api_port_crc_params_set")
crc_params = sx_port_crc_params_t()
crc_params.bad_crc_ingress_mode = SX_PORT_BAD_CRC_INGRESS_MODE_DROP
crc_params.crc_egress_recalc_mode = SX_PORT_CRC_EGRESS_RECALC_MODE_ALLOW
sx_port_crc_params_t_p_assign(crc_params_p, crc_params)
rc = sx_api_port_crc_params_set(handle, log_port, crc_params_p)
print("sx_api_port_crc_params_set return %d" % (rc))
if rc != 0:
    sys.exit(errno.EACCES)
