#!/usr/bin/env python
'''
This file contains Python command example for the PORT module.
Python commands syntax is very similar to the Switch SDK APIs.
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below demonstrates the configuration of rate for 8x lanes port.
logic of the example is the following:
- Reset port API type usage
- Get the current port mapping configurations
- Reset port API type usage
'''
import sys
import errno
import pdb
import time
from python_sdk_api.sx_api import *
from python_sdk_api.sxd_api import *
from test_infra_common import *
import argparse
import pdb

######################################################
#    defines
######################################################
SWID = 0
DEVICE_ID = 1
MAX_PORTS = 64


def auto_int(x):
    return int(x, 0)


ERR_FILE_LOCATION = '/tmp/python_err_log.txt'

if len(sys.argv) < 3:
    print "Bad parameters."
    print "      " + sys.argv[0] + " --pass_routed_frames < 0 | 1 >"

    sys.exit(0)

parser = argparse.ArgumentParser(description='sx_api_port_isolate_mode_set API')
parser.add_argument('--pass_routed_frames', type=int, help='pass_routed_frames: boolean')
args = parser.parse_args()

print "Value to be set for pass_routed_frames: %d" % args.pass_routed_frames

# file_exist = os.path.isfile(ERR_FILE_LOCATION)
# sys.stderr = open(ERR_FILE_LOCATION, 'w')
# if not file_exist:
#     os.chmod(ERR_FILE_LOCATION, 0o777)

######################################################
#    functions
######################################################


def get_chip_type_and_rev():
    rc = sxd_access_reg_init(0, None, 4)
    if (rc != 0):
        print ("Failed to initializing register access.\nPlease check that SDK is running.")
        sys.exit(errno.EACCES)
    mgir = ku_mgir_reg()
    meta = sxd_reg_meta_t()
    meta.dev_id = 1
    meta.swid = SWID
    meta.access_cmd = SXD_ACCESS_CMD_GET
    rc = sxd_access_reg_mgir(mgir, meta, 1, None, None)
    assert 0 == rc
    return mgir.hw_info.device_id, mgir.hw_info.device_hw_revision


def main():
    #     pdb.set_trace()
    print "[+] opening sdk"
    rc, handle = sx_api_open(None)
    print ("sx_api_open handle:0x%x , rc %d " % (handle, rc))
    if (rc != SX_STATUS_SUCCESS):
        print "Failed to open api handle.\nPlease check that SDK is running."
        sys.exit(errno.EACCES)

   # Check the current chip type
    device_id, device_hw_revision = get_chip_type_and_rev()
    if device_id != SXD_MGIR_HW_DEV_ID_SPECTRUM and device_id != SXD_MGIR_HW_DEV_ID_SPECTRUM2:
        print "The following example is suitable only for Spectrum chip"
        sys.exit(errno.EACCES)

    sx_port_isolate_mode_p = new_sx_port_isolate_mode_t_p()
    sx_port_isolate_mode = sx_port_isolate_mode_t()
    sx_port_isolate_mode.pass_routed_frames = args.pass_routed_frames
    sx_port_isolate_mode_t_p_assign(sx_port_isolate_mode_p, sx_port_isolate_mode)

    rc = sx_api_port_isolate_mode_set(handle, sx_port_isolate_mode_p)
    assert rc == SX_STATUS_SUCCESS, "sx_api_port_isolate_set failed. rc:%d" % rc

    rc = sx_api_port_isolate_mode_get(handle, sx_port_isolate_mode_p)
    assert rc == SX_STATUS_SUCCESS, "sx_api_port_isolate_set failed. rc:%d" % rc

    sx_port_isolate_mode = sx_port_isolate_mode_t_p_value(sx_port_isolate_mode_p)
    print "New value of sx_port_isolate_mode: %d" % sx_port_isolate_mode.pass_routed_frames

    sx_api_close(handle)
    print("Success.")


################################################################################
#                             Main                                             #
################################################################################
if __name__ == "__main__":
    main()
