#!/usr/bin/env python
'''
This file contains Python command example for the PORT module.
Python commands syntax is very similar to the Switch SDK APIs.
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below demonstrates the configuration of rate for 8x lanes port.
logic of the example is the following:
- Reset port API type usage
- Get the current port mapping configurations
- Reset port API type usage
'''
import sys
import errno
import pdb
import time
from python_sdk_api.sx_api import *
from python_sdk_api.sxd_api import *
from test_infra_common import *
import argparse

######################################################
#    defines
######################################################
SWID = 0
DEVICE_ID = 1
MAX_PORTS = 64


def auto_int(x):
    return int(x, 0)


ERR_FILE_LOCATION = '/tmp/python_err_log.txt'

if len(sys.argv) < 5:
    print "Bad parameters."
    print "    " + sys.argv[0] + " --cmd <add(1)|set(15)|delete(3)|delete_all(4)> --egress_port 0x10100  --ingress_ports 0x10300 0x10500"

    sys.exit(0)

parser = argparse.ArgumentParser(description='sx_api_port_isolation_set API')
# parser.add_argument('--cmd', type=int, choices=[SX_ACCESS_CMD_ADD,SX_ACCESS_CMD_SET,SX_ACCESS_CMD_DELETE,SX_ACCESS_CMD_DELETE_ALL],
#                     help='Command: add=SX_ACCESS_CMD_ADD|set=SX_ACCESS_CMD_SET|delete=SX_ACCESS_CMD_DELETE|delete_all=SX_ACCESS_CMD_DELETE_ALL')
parser.add_argument('--cmd', type=str, choices=["ADD", "add", "1", "SET", "set", "15", "DELETE", "delete", "3", "DELETE_ALL", "delete_all", "4"],
                    help='Command: ADD|SET|DELETE|DELETE_ALL')
parser.add_argument('--egress_port', type=auto_int, help='Egress port ')
parser.add_argument('--ingress_ports', type=auto_int, nargs='+', help='Ingress ports')
args = parser.parse_args()

cmd_dict = {
    "ADD": SX_ACCESS_CMD_ADD,
    "SET": SX_ACCESS_CMD_SET,
    "DELETE": SX_ACCESS_CMD_DELETE,
    "DELETE_ALL": SX_ACCESS_CMD_DELETE_ALL,
    "add": SX_ACCESS_CMD_ADD,
    "set": SX_ACCESS_CMD_SET,
    "delete": SX_ACCESS_CMD_DELETE,
    "delete_all": SX_ACCESS_CMD_DELETE_ALL,
    "1": SX_ACCESS_CMD_ADD,
    "15": SX_ACCESS_CMD_SET,
    "3": SX_ACCESS_CMD_DELETE,
    "4": SX_ACCESS_CMD_DELETE_ALL,
}

print "Cmd: %s (%d)" % (args.cmd, cmd_dict[args.cmd])
print "Egress port: 0x%x" % args.egress_port
if args.ingress_ports:
    for i, port in enumerate(args.ingress_ports):
        print ("Ingress ports[%d]: 0x%x" % (i, port))

# file_exist = os.path.isfile(ERR_FILE_LOCATION)
# sys.stderr = open(ERR_FILE_LOCATION, 'w')
# if not file_exist:
#     os.chmod(ERR_FILE_LOCATION, 0o777)

######################################################
#    functions
######################################################


def get_chip_type_and_rev():
    rc = sxd_access_reg_init(0, None, 4)
    if (rc != 0):
        print ("Failed to initializing register access.\nPlease check that SDK is running.")
        sys.exit(errno.EACCES)
    mgir = ku_mgir_reg()
    meta = sxd_reg_meta_t()
    meta.dev_id = 1
    meta.swid = SWID
    meta.access_cmd = SXD_ACCESS_CMD_GET
    rc = sxd_access_reg_mgir(mgir, meta, 1, None, None)
    assert 0 == rc
    return mgir.hw_info.device_id, mgir.hw_info.device_hw_revision


def port_isolate_get(handle):
    ingress_ports_cnt_p = new_uint32_t_p()
    uint32_t_p_assign(ingress_ports_cnt_p, MAX_PORTS)
    ingress_ports_arr = new_sx_port_log_id_t_arr(MAX_PORTS)
    for i in range(MAX_PORTS):
        sx_port_log_id_t_arr_setitem(ingress_ports_arr, i, 0)
    rc = sx_api_port_isolate_get(handle, args.egress_port, ingress_ports_arr, ingress_ports_cnt_p)
    assert rc == SX_STATUS_SUCCESS, "sx_api_port_isolate_get failed. [%d]" % rc

    ingress_ports_cnt = uint32_t_p_value(ingress_ports_cnt_p)
    print "GET "
    print "    Egress port:0x%x " % (args.egress_port)
    print "    Ingress port cnt:%d " % (ingress_ports_cnt)
    for i in range(ingress_ports_cnt):
        port = sx_port_log_id_t_arr_getitem(ingress_ports_arr, i)
        print "    ingress port[%d]:0x%x " % (i, port)


def main():
    #     pdb.set_trace()
    print "[+] opening sdk"
    rc, handle = sx_api_open(None)
    print ("sx_api_open handle:0x%x , rc %d " % (handle, rc))
    if (rc != SX_STATUS_SUCCESS):
        print "Failed to open api handle.\nPlease check that SDK is running."
        sys.exit(errno.EACCES)

   # Check the current chip type
    device_id, device_hw_revision = get_chip_type_and_rev()
    if device_id != SXD_MGIR_HW_DEV_ID_SPECTRUM and device_id != SXD_MGIR_HW_DEV_ID_SPECTRUM2:
        print "The following example is suitable only for Spectrum chip"
        sys.exit(errno.EACCES)

    port_isolate_get(handle)

    ingress_ports_cnt = 0
    if args.ingress_ports:
        ingress_ports_cnt = len(args.ingress_ports)
    ingress_ports_arr = new_sx_port_log_id_t_arr(MAX_PORTS)
    for i in range(MAX_PORTS):
        sx_port_log_id_t_arr_setitem(ingress_ports_arr, i, 0)

    print "SET"
    print "    Egress port:0x%x " % args.egress_port
    print "    Ingress port cnt:%d " % (ingress_ports_cnt)

    if args.ingress_ports:
        for i, port in enumerate(args.ingress_ports):
            sx_port_log_id_t_arr_setitem(ingress_ports_arr, i, port)
            print ("    Ingress_ports[%d]=0x%x " % (i, port))

    rc = sx_api_port_isolate_set(handle, cmd_dict[args.cmd], args.egress_port, ingress_ports_arr, ingress_ports_cnt)
    assert rc == SX_STATUS_SUCCESS, "sx_api_port_isolate_set failed. [%d]" % rc

    port_isolate_get(handle)

    sx_api_close(handle)
    print("Success.")


################################################################################
#                             Main                                             #
################################################################################
if __name__ == "__main__":
    main()
