#!/usr/bin/env python
'''
This file contains Python command example for the PORT module.
Python commands syntax is very similar to the Switch SDK APIs.
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below demonstrates the configuration of different port attributes.
'''
import sys
import errno
from python_sdk_api.sx_api import *
from test_infra_common import *

SPECTRUM_SWID = 0
PRIO = 6
DEFAULT_EGRESS_POOL = 11
DEFAULT_INGRESS_POOL = 0

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % (handle, rc))
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)
port_list = get_ports(handle, 2)
PORT1 = port_list[0]
PORT2 = port_list[1]


def allocate_vlan_port_list(ports_dict):
    " This function allocates sx_vlan_ports_t array, out of port dictionary. "

    port_list = new_sx_vlan_ports_t_arr(len(ports_dict))
    for i, port_id in enumerate(ports_dict.keys()):
        vlan_port = sx_vlan_ports_t()
        vlan_port.log_port = port_id
        vlan_port.is_untagged = ports_dict[port_id]
        sx_vlan_ports_t_arr_setitem(port_list, i, vlan_port)
    return port_list


def add_ports_to_vlan(vlan_id, ports_dict):
    " This function adds speificed ports_dict into target vlan. "
    print ("--------------- ADD PORT VLAN MEMBERSHIP  ------------------------------")

    port_list = allocate_vlan_port_list(ports_dict)
    rc = sx_api_vlan_ports_set(handle, SX_ACCESS_CMD_ADD, SPECTRUM_SWID, vlan_id, port_list, len(ports_dict))
    assert SX_STATUS_SUCCESS == rc, "Failed to add port %s to vlan %d" % (str(ports_dict.keys()), vlan_id)
    for port in ports_dict:
        print "Added port 0x%x to vlan %d, rc: %d" % (port, vlan_id, rc)


add_ports_to_vlan(1, {PORT1: SX_TAGGED_MEMBER})
add_ports_to_vlan(1, {PORT2: SX_TAGGED_MEMBER})

""" Set FDB UC for traffic """
mac_list_p = new_sx_fdb_uc_mac_addr_params_t_arr(2)
data_cnt_p = new_uint32_t_p()
uint32_t_p_assign(data_cnt_p, 2)
mac_addr = ether_addr("24:8a:07:55:11:1c")

mac_entry1 = sx_fdb_uc_mac_addr_params_t()
mac_entry1.mac_addr = mac_addr
mac_entry1.fid_vid = 1          # Filtering Identifier, VID for static MAC
mac_entry1.log_port = PORT1
mac_entry1.entry_type = SX_FDB_UC_STATIC
mac_entry1.action = SX_FDB_ACTION_FORWARD
sx_fdb_uc_mac_addr_params_t_arr_setitem(mac_list_p, 0, mac_entry1)

mac_addr = ether_addr("24:8a:07:55:11:1d")
mac_entry2 = sx_fdb_uc_mac_addr_params_t()
mac_entry2.mac_addr = mac_addr
mac_entry2.fid_vid = 1          # Filtering Identifier, VID for static MAC
mac_entry2.log_port = PORT2
mac_entry2.entry_type = SX_FDB_UC_STATIC
mac_entry2.action = SX_FDB_ACTION_FORWARD
sx_fdb_uc_mac_addr_params_t_arr_setitem(mac_list_p, 1, mac_entry2)

rc = sx_api_fdb_uc_mac_addr_set(handle, SX_ACCESS_CMD_ADD, 0, mac_list_p, data_cnt_p)
data_cnt = uint32_t_p_value(data_cnt_p)
print("sx_api_fdb_uc_mac_addr_set added %d enties, rc: %d " % (data_cnt, rc))
if rc != 0:
    sys.exit(errno.EACCES)

""" Enable PFC """
fc_mode = SX_PORT_FLOW_CTRL_MODE_TX_EN_RX_EN

rc = sx_api_port_pfc_enable_set(handle, PORT1, PRIO, fc_mode)
print ("sx_api_port_pfc_enable_set: 0x%x enabled, rc %d" % (PORT1, rc))
if rc != 0:
    sys.exit(errno.EACCES)

#fc_mode = SX_PORT_FLOW_CTRL_MODE_TX_DIS_RX_DIS
rc = sx_api_port_pfc_enable_set(handle, PORT2, PRIO, fc_mode)
print ("sx_api_port_pfc_enable_set: 0x%x enabled, rc %d" % (PORT2, rc))
if rc != 0:
    sys.exit(errno.EACCES)

""" Enable TQ.TC """
attr_p = new_sx_cos_port_buffer_attr_t_p()

attr_p.type = SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E
attr_p.attr.egress_port_tc_buff_attr.size = 0
attr_p.attr.egress_port_tc_buff_attr.tc = PRIO
attr_p.attr.egress_port_tc_buff_attr.pool_id = DEFAULT_EGRESS_POOL

rc = sx_api_cos_port_buff_type_set(handle, SX_ACCESS_CMD_SET, PORT2, attr_p, 1)
print ("sx_api_cos_port_buff_type_set [cmd=%d, log_port=0x%x , cnt=%d, rc=%d] " % (SX_ACCESS_CMD_SET, PORT2, 1, rc))
if rc != 0:
    sys.exit(errno.EACCES)

shared_attr_p = new_sx_cos_port_shared_buffer_attr_t_p()
shared_attr_p.type = SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E
shared_attr_p.attr.egress_port_tc_shared_buff_attr.tc = PRIO
shared_attr_p.attr.egress_port_tc_shared_buff_attr.max.mode = SX_COS_BUFFER_MAX_MODE_DYNAMIC_E
shared_attr_p.attr.egress_port_tc_shared_buff_attr.max.max.alpha = SX_COS_PORT_BUFF_ALPHA_2_E
shared_attr_p.attr.egress_port_tc_shared_buff_attr.pool_id = DEFAULT_EGRESS_POOL

rc = sx_api_cos_port_shared_buff_type_set(handle, SX_ACCESS_CMD_SET, PORT2, shared_attr_p, 1)
print ("sx_api_cos_port_buff_type_set [cmd=%d, log_port=0x%x , cnt=%d, rc=%d] " % (SX_ACCESS_CMD_SET, PORT2, 1, rc))
if rc != 0:
    sys.exit(errno.EACCES)

""" Set LOSSLESS """
attr_p = new_sx_cos_port_buffer_attr_t_p()

attr_p.type = SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E
attr_p.attr.ingress_port_pg_buff_attr.size = 1000
attr_p.attr.ingress_port_pg_buff_attr.pg = 0
""" is_lossy: 0=Lossless, 1=Lossy """
attr_p.attr.ingress_port_pg_buff_attr.is_lossy = 0
attr_p.attr.ingress_port_pg_buff_attr.xon = 0
attr_p.attr.ingress_port_pg_buff_attr.xoff = 500
attr_p.attr.ingress_port_pg_buff_attr.pool_id = DEFAULT_INGRESS_POOL

rc = sx_api_cos_port_buff_type_set(handle, SX_ACCESS_CMD_SET, PORT1, attr_p, 1)
print ("sx_api_cos_port_buff_type_set [cmd=%d, log_port=0x%x , cnt=%d, rc=%d] " % (SX_ACCESS_CMD_SET, PORT1, 1, rc))
if rc != 0:
    sys.exit(errno.EACCES)

shared_attr_p = new_sx_cos_port_shared_buffer_attr_t_p()
shared_attr_p.type = SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E
shared_attr_p.attr.ingress_port_pg_shared_buff_attr.pg = 0
shared_attr_p.attr.ingress_port_pg_shared_buff_attr.max.mode = SX_COS_BUFFER_MAX_MODE_DYNAMIC_E
shared_attr_p.attr.ingress_port_pg_shared_buff_attr.max.max.alpha = SX_COS_PORT_BUFF_ALPHA_1_128_E
shared_attr_p.attr.ingress_port_pg_shared_buff_attr.pool_id = DEFAULT_INGRESS_POOL

rc = sx_api_cos_port_shared_buff_type_set(handle, SX_ACCESS_CMD_SET, PORT1, shared_attr_p, 1)
print ("sx_api_cos_port_buff_type_set [cmd=%d, log_port=0x%x , cnt=%d, rc=%d] " % (SX_ACCESS_CMD_SET, PORT1, 1, rc))
if rc != 0:
    sys.exit(errno.EACCES)
