#!/usr/bin/env python
'''
This file contains Python command example for the PORT module.
Python commands syntax is very similar to the Switch SDK APIs.
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below demonstrates the configuration of different port attributes.
'''
import sys
import errno
from python_sdk_api.sx_api import *
import argparse

CMD_SET = 1
LOOPBACK_DICT = {0: 'DISABLED', 1: 'EXTERNAL', 2: 'INTERNAL', 3: 'BOTH', 4: 'INTERNAL_LR'}


def auto_int(x):
    return int(x, 0)


if len(sys.argv) < 4 and len(sys.argv) != 1:
    print "Bad parameters."
    print "Example:" + sys.argv[0] + "--cmd 1 --log_port 0x10100 --loopback_type 2"
    sys.exit(errno.EINVAL)

parser = argparse.ArgumentParser(description='Port loopback')
parser.add_argument('--cmd', default=0, type=int, help="get(0), set(1)")
parser.add_argument('--log_port', default=0x10100, type=auto_int, help='Logical port ID')
parser.add_argument('--loopback_type', default=0, type=int, help='disable(0), external(1), internal(2), both(3), internal_lr(4)')
args = parser.parse_args()

if len(sys.argv) == 1:
    print "Note that for the current example will be used default parameters:\n\tlog_port: 0x%x \n\tloopback_type: %d" % (args.log_port, args.loopback_type)

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % (handle, rc))
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

log_port = args.log_port

if (args.cmd == CMD_SET):
    print "[+] Loopback set: log_port:0x%x loopback:%s(%d)" % (log_port, LOOPBACK_DICT[args.loopback_type], args.loopback_type)
    rc = sx_api_port_state_set(handle, log_port, SX_PORT_ADMIN_STATUS_DOWN)
    print ("sx_api_port_state_set log_port 0x%x Down, rc %d " % (log_port, rc))
    rc = sx_api_port_phys_loopback_set(handle, log_port, args.loopback_type)
    print ("sx_api_port_phys_loopback_set log_port 0x%x type %d, rc %d " % (log_port, args.loopback_type, rc))
    rc = sx_api_port_state_set(handle, log_port, SX_PORT_ADMIN_STATUS_UP)
    print ("sx_api_port_state_set log_port 0x%x Up, rc %d " % (log_port, rc))

print "[+] Loopback get"
loopback_type_p = new_sx_port_phys_loopback_t_p()
sx_port_phys_loopback_t_p_assign(loopback_type_p, args.loopback_type)
rc = sx_api_port_phys_loopback_get(handle, log_port, loopback_type_p)
loopback_type = sx_port_phys_loopback_t_p_value(loopback_type_p)
print ("sx_api_port_phys_loopback_get log_port 0x%x loopback_type:%s(%d), rc %d " %
       (log_port, LOOPBACK_DICT[loopback_type], loopback_type, rc))

sx_api_close(handle)
