#!/usr/bin/env python
'''
This file contains Python command example for the PORT module.
Python commands syntax is very similar to the Switch SDK APIs.
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below demonstrates the configuration port split / uspilt flow.
'''

import sys
import errno
import pdb
import time
from python_sdk_api.sx_api import *
from python_sdk_api.sxd_api import *
from test_infra_common import *
import argparse

######################################################
#    Defines
######################################################
SWID = 0
DEVICE_ID = 1

ERR_FILE_LOCATION = '/tmp/python_err_log.txt'
file_exist = os.path.isfile(ERR_FILE_LOCATION)
sys.stderr = open(ERR_FILE_LOCATION, 'w')
if not file_exist:
    os.chmod(ERR_FILE_LOCATION, 0o777)


######################################################
#    Handle input params
######################################################

def help_cmd():
    print ("\n")
    print (sys.argv)
    print "Bad parameters."
    print "sx_api_port_split.py --old_ports <op> --new_ports <np> --new_modules <m> --new_lane_bmaps <bmap> --new_width <w> [--rate <r>]"
    print "Supported width: 1 | 2 | 4 | 8,"
    print "Supported rate: 100M | 1G | 10G | 25G | 40G | 50G | 100G | 200G | 400G | auto"
    print ("\n")
    print "Example 1 for split 2x (MSN2700-SPC1):" + sys.argv[0] + \
        " --old_ports 0x10100 --new_ports 0x10100 0x10200 --new_modules 17 17 --new_lane_bmaps 0x03 0x0C --new_width 2"
    print "Example 1 for unsplit, after split 2x (MSN2700-SPC1):" + sys.argv[0] + \
        " --old_ports 0x10100 0x10200 --new_ports 0x10100 --new_modules 17 --new_lane_bmaps 0x0F --new_width 4"
    print "Example 2 for split 4x (MSN2700-SPC1):" + sys.argv[0] + \
        " --old_ports 0x11100 --new_ports 0x11100 0x11200 0x11300 0x11400 --new_modules 25 25 25 25  --new_lane_bmaps 0x1 0x2 0x4 0x8 --new_width 1"
    print "Example 2 for unsplit, after split 4x (MSN2700-SPC1):" + sys.argv[0] + \
        " --old_ports 0x11100 0x11200 0x11300 0x11400 --new_ports 0x11100 0x11300 --new_modules 25 26  --new_lane_bmaps 0xF 0xF --new_width 4"
    print ("\n")
    print "Example 3 for split 2x (MSN3700-SPC2):" + sys.argv[0] + \
        " --old_ports 0x10100 --new_ports 0x10100 0x10300 --new_modules 17 17 --new_lane_bmaps 0x03 0x0C --new_width 2 --rate 50G"
    print "Example 3 for unsplit, after split 2x (MSN3700-SPC2):" + sys.argv[0] + \
        " --old_ports 0x10100 0x10300 --new_ports 0x10100 --new_modules 17 --new_lane_bmaps 0x0F --new_width 4 -rate auto"
    print "Example 4 for split 4x (MSN3700-SPC2):" + sys.argv[0] + \
        " --old_ports 0x10100 --new_ports 0x10100 0x10200 0x10300 0x10400 --new_modules 17 17 17 17 --new_lane_bmaps 0x1 0x2 0x4 0x8 --new_width 1 --rate 25G"
    print "Example 4 for unsplit, after split 4x (MSN3700-SPC2):" + sys.argv[0] + \
        " --old_ports 0x10100 0x10200 0x10300 0x10400 --new_ports 0x10100 --new_modules 17 --new_lane_bmaps 0x0F --new_width 4 --rate auto"
    print ("\n")
    print "Example 5 for split 4x (MSN4700-SPC3):" + sys.argv[0] + \
        " --old_ports 0x10100 --new_ports 0x10100 0x10300 --new_modules 18 18 --new_lane_bmaps 0x0F 0xF0 --new_width 4 --rate 100G"
    print "Example 5 for unsplit, after split 4x (MSN4700-SPC3):" + sys.argv[0] + \
        " --old_ports 0x10100 0x10300 --new_ports 0x10100 --new_modules 18 --new_lane_bmaps 0xFF --new_width 8 --rate auto"

    print "Example 6 for split 2x (MSN4700-SPC3):" + sys.argv[0] + \
        " --old_ports 0x10100 --new_ports 0x10100 0x10200 0x10300 0x10400 --new_modules 18 18 18 18 --new_lane_bmaps 0x03 0x0C 0x30 0xC0 --new_width 2 --rate 50G"
    print "Example 6 for unsplit, after split 2x (MSN4700-SPC3):" + sys.argv[0] + \
        " --old_ports 0x10100 0x10200 0x10300 0x10400 --new_ports 0x10100 --new_modules 18 --new_lane_bmaps 0xFF --new_width 8 --rate auto"

    print "Example 7 for split 1x (MSN4700-SPC3):" + sys.argv[0] + \
        " --old_ports 0x10100 --new_ports 0x10100 0x10200 0x10300 0x10400 0x10500 0x10600 0x10700 0x10800 --new_modules 18 18 18 18 18 18 18 18 --new_lane_bmaps 0x01 0x02 0x04 0x08 0x10 0x20 0x40 0x80 --new_width 1 --rate 50G"
    print "Example 7 for unsplit, after split 1x (MSN4700-SPC3):" + sys.argv[0] + \
        " --old_ports 0x10100 0x10200 0x10300 0x10400 0x10500 0x10600 0x10700 0x10800 --new_ports 0x10100 0x10500 --new_modules 18 17 --new_lane_bmaps 0xFF 0xFF --new_width 8 --rate auto"

    print ("\n")
    print "IMPORTANT: the examples are for MSN2700-SPC1 / MSN3700-SPC2 / MSN4700-SPC3. For other systems and other ports please check module numbers."
    print "IMPORTANT: Note the difference for split_2x in SPC2 - new port after the split is port+2, not port+1 as in SPC1"
    print "IMPORTANT: rate option is not supported by SPC-1 systems"
    sys.exit(0)

if len(sys.argv) < 8:
    help_cmd()


def auto_int(x):
    return int(x, 0)


parser = argparse.ArgumentParser(description='Port Split utility')
parser.add_argument('--old_ports', type=auto_int, nargs='+', help='Old logical ports - before split/unsplit')
parser.add_argument('--new_ports', type=auto_int, nargs='+', help='New logical ports - after split/unsplit')
parser.add_argument('--new_modules', type=auto_int, nargs='+', help='Modules per new port')
parser.add_argument('--new_lane_bmaps', type=auto_int, nargs='+', help='Lane bitmask per new port')
parser.add_argument('--new_width', default=1, type=int, help='Number of lanes for new ports')
parser.add_argument('--rate', default=['auto'], type=str, nargs='+', help='New port rate value')
args = parser.parse_args()

valid_rates = {"100M", "1G", "10G", "25G", "40G", "50G", "100G", "200G", "400G", "auto"}
admin_rate = args.rate[0].split(",")
for rate in admin_rate:
    if rate not in valid_rates:
        print ("Specified %s rate value is not valid." % (rate))
        help_cmd()
        sys.exit(1)

for i, port in enumerate(args.old_ports):
    print ("Old ports[%d]: 0x%x" % (i, port))

for i, port in enumerate(args.new_ports):
    print ("New ports[%d]: 0x%x" % (i, port))

for i, module in enumerate(args.new_modules):
    print ("New modules[%d]: %d" % (i, module))

for i, lane_bmap in enumerate(args.new_lane_bmaps):
    print ("New lane_bmaps[%d]: 0x%x" % (i, lane_bmap))

print "New width:" + str(args.new_width)

######################################################
#    Local functions
######################################################

def get_device_id_and_rev():
    mgir = ku_mgir_reg()
    meta = sxd_reg_meta_t()
    meta.dev_id = 1
    meta.swid = SWID
    meta.access_cmd = SXD_ACCESS_CMD_GET

    rc = sxd_access_reg_mgir(mgir, meta, 1, None, None)
    assert 0 == rc

    return mgir.hw_info.device_id, mgir.hw_info.device_hw_revision


def get_chip_type():
    print ("Retrieving Chip Type from SDK")
    device_id, _ = get_device_id_and_rev()

    if device_id == 0xCB84:
        print ("Running over SPECTRUM chip")
        _chip_type = SX_CHIP_TYPE_SPECTRUM
    elif device_id == 0xCF6C:
        print ("Running over SPECTRUM2 chip")
        _chip_type = SX_CHIP_TYPE_SPECTRUM2
    elif device_id == 0xCF70:
        print ("Running over SPECTRUM3 chip")
        _chip_type = SX_CHIP_TYPE_SPECTRUM3
    else:
        print ("--ERROR-- chip type is not supported!")
        sys.exit(errno.EACCES)

    return _chip_type


def port_state_set(handle, log_port, admin_state):
    rc = sx_api_port_state_set(handle, log_port, admin_state)
    assert rc == SX_STATUS_SUCCESS, "Failed to change port state."


def port_init_handle(handle, port, is_init=True):
    if is_init:
        rc = sx_api_port_init_set(handle, port)
        assert rc == SX_STATUS_SUCCESS, "Failed to init port"
    else:
        rc = sx_api_port_deinit_set(handle, port)
        assert rc == SX_STATUS_SUCCESS, "Failed to de-init port"


# de-init and init port to reset port speed API type flag
def reset_port_type_api_flag(handle, ports_lst):
    for port in ports_lst:
        swid = port_swid_bind_get(handle, port)
        if swid != SWID:
            continue
        port_init_handle(handle, port, is_init=False)
        port_init_handle(handle, port, is_init=True)


def port_swid_bind_set(handle, port, swid):
    rc = sx_api_port_swid_bind_set(handle, port, swid)
    assert rc == SX_STATUS_SUCCESS, "Failed to bind port."


def port_swid_bind_get(handle, port):
    sx_swid_p = new_sx_swid_t_p()
    rc = sx_api_port_swid_bind_get(handle, port, sx_swid_p)
    assert rc == SX_STATUS_SUCCESS, "Failed to bind port."
    swid = sx_swid_t_p_value(sx_swid_p)
    return swid


def port_unmapping_set(handle, port, log_port_cnt=1):
    log_port_list_arr = new_sx_port_log_id_t_arr(log_port_cnt)
    sx_port_log_id_t_arr_setitem(log_port_list_arr, 0, port)

    port_mapping_list_p = new_sx_port_mapping_t_arr(log_port_cnt)
    mapping_item = new_sx_port_mapping_t_arr(1)

    for i in range(0, log_port_cnt):
        mapping_item = sx_port_mapping_t_arr_getitem(port_mapping_list_p, i)
        mapping_item.local_port = ((port & 0x0000FF00) >> 8)
        mapping_item.mapping_mode = SX_PORT_MAPPING_MODE_DISABLE
        mapping_item.module_port = 0
        mapping_item.width = 0
        mapping_item.lane_bmap = 0
        mapping_item.config_hw = 0

    sx_port_mapping_t_arr_setitem(port_mapping_list_p, 0, mapping_item)

    rc = sx_api_port_mapping_set(handle, log_port_list_arr, port_mapping_list_p, log_port_cnt)
    assert rc == SX_STATUS_SUCCESS, "Failed to disable mapping."


def port_mapping_attr_set(handle, port, module, lane_bmap):
    print ("port_mapping_attr_set: port:0x%x module:%d lane_bmap:0x%x" % (port, module, lane_bmap))
    log_port_list_arr = new_sx_port_log_id_t_arr(1)
    sx_port_log_id_t_arr_setitem(log_port_list_arr, 0, port)

    port_mapping_list_p = new_sx_port_mapping_t_arr(1)
    mapping_item = new_sx_port_mapping_t_arr(1)

    mapping_item = sx_port_mapping_t_arr_getitem(port_mapping_list_p, 0)
    mapping_item.local_port = ((port & 0x0000FF00) >> 8)
    mapping_item.mapping_mode = SX_PORT_MAPPING_MODE_ENABLE
    mapping_item.width = args.new_width
    mapping_item.config_hw = 0
    mapping_item.module_port = module - 1
    mapping_item.lane_bmap = lane_bmap

    sx_port_mapping_t_arr_setitem(port_mapping_list_p, 0, mapping_item)

    rc = sx_api_port_mapping_set(handle, log_port_list_arr, port_mapping_list_p, 1)
    assert rc == SX_STATUS_SUCCESS, "Failed to configure port mapping."


def ports_mapping_attr_get(handle, port):
    log_port_list_arr = new_sx_port_log_id_t_arr(1)
    port_mapping_list_arr = new_sx_port_mapping_t_arr(1)
    port_cnt = 1

    sx_port_log_id_t_arr_setitem(log_port_list_arr, 0, port)
    rc = sx_api_port_mapping_get(handle, log_port_list_arr, port_mapping_list_arr, port_cnt)
    assert rc == SX_STATUS_SUCCESS, "Failed to get port mapping attributes."

    port_mapping_att = sx_port_mapping_t_arr_getitem(port_mapping_list_arr, 0)
    return port_mapping_att.module_port, port_mapping_att.width, port_mapping_att.lane_bmap


def rstp_port_state_set(handle, log_port, mstp_state):
    rc = sx_api_rstp_port_state_set(handle, log_port, mstp_state)
    assert rc == SX_STATUS_SUCCESS, "Failed to configure RSTP port state."


def add_ports_to_vlan(handle, vlan_id, ports_dict):
    port_list = new_sx_vlan_ports_t_arr(len(ports_dict))
    for i, port_id in enumerate(ports_dict.keys()):
        vlan_port = sx_vlan_ports_t()
        vlan_port.log_port = port_id
        vlan_port.is_untagged = ports_dict[port_id]
        sx_vlan_ports_t_arr_setitem(port_list, i, vlan_port)

    rc = sx_api_vlan_ports_set(handle, SX_ACCESS_CMD_ADD, SWID, vlan_id, port_list, len(ports_dict))
    assert rc == SX_STATUS_SUCCESS, "Failed to add port into the VLAN."


def port_admin_speed_10G_set(handle, port):
    print ("Setting port rate for port 0x%x\n" % (port))
    admin_speed_p = new_sx_port_speed_capability_t_p()
    admin_speed_p.mode_10GB_CR = 1
    admin_speed_p.mode_10GB_CX = 1
    admin_speed_p.mode_10GB_SR = 1
    admin_speed_p.mode_10GB_KR = 1
    admin_speed_p.mode_10GB_ER_LR = 1
    admin_speed_p.mode_auto = 0
    admin_speed_p.force = 0

    rc = sx_api_port_speed_admin_set(handle, port, admin_speed_p)
    assert SX_STATUS_SUCCESS == rc
    print ("[+] 10G speed is configured for log_port: 0x%x." % (port))


def enable_rate(rate, admin_rate):
    return (True if (rate in admin_rate) else False)


def port_rate_configuration(handle, port, admin_rate):
    # Went through the admin_rate list and enable appropriate values
    rate_bitmask_p = new_sx_port_rate_bitmask_t_p()
    rate_bitmask_p.rate_100M = enable_rate("100M", admin_rate)
    rate_bitmask_p.rate_1G   = enable_rate("1G", admin_rate)
    rate_bitmask_p.rate_10G  = enable_rate("10G", admin_rate)
    rate_bitmask_p.rate_25G  = enable_rate("25G", admin_rate)
    rate_bitmask_p.rate_40G  = enable_rate("40G", admin_rate)
    rate_bitmask_p.rate_50G  = enable_rate("50G", admin_rate)
    rate_bitmask_p.rate_100G = enable_rate("100G", admin_rate)
    rate_bitmask_p.rate_200G = enable_rate("200G", admin_rate)
    rate_bitmask_p.rate_400G = enable_rate("400G", admin_rate)
    rate_bitmask_p.rate_auto = enable_rate("auto", admin_rate)
    rate_bitmask_p.force = False  # Note that this option may be enabled only for single rate value

    # Set all module types
    phy_type_p = new_sx_port_phy_module_type_bitmask_t_p()
    phy_type_p.module_smf_up_500m = True
    phy_type_p.module_smf_up_2km = True
    phy_type_p.module_smf_above_2km = True
    phy_type_p.module_mmf_up_100m = True
    phy_type_p.module_mmf_above_100m = True
    phy_type_p.module_aoc_acc_up_30m = True
    phy_type_p.module_aoc_acc_above_30m = True
    phy_type_p.module_base_cr = True
    phy_type_p.module_base_tp = True

    module_id, _, _ = ports_mapping_attr_get(handle, port)

    rc = sx_api_port_phy_module_type_set(handle, module_id, phy_type_p)
    assert rc == SX_STATUS_SUCCESS, "Failed to set port phy type."

    rc = sx_api_port_rate_set(handle, port, rate_bitmask_p)
    assert rc == SX_STATUS_SUCCESS, "Failed to set port admin rate."

    print ("[+] Rate and PMD type is configured for log_port: 0x%x." % (port))


def split_unsplit_ports(handle, chip_type, old_ports_list, new_ports_list):
    un_bind_swid = 255

    for port in old_ports_list:
        swid = port_swid_bind_get(handle, port)
        print("Old port:0x%x swid:%d" % (port, swid))
        if swid != SWID:
            continue

        # ports must be in DOWN state
        port_state_set(handle, port, SX_PORT_ADMIN_STATUS_DOWN)
        # de-init ports
        port_init_handle(handle, port, is_init=False)
        # un-bind ports
        port_swid_bind_set(handle, port, un_bind_swid)
        # disable mapping
        port_unmapping_set(handle, port)

    for port in new_ports_list:
        swid = port_swid_bind_get(handle, port)
        print("New port:0x%x swid:%d" % (port, swid))
        if swid != SWID:
            continue

        # ports must be in DOWN state
        port_state_set(handle, port, SX_PORT_ADMIN_STATUS_DOWN)
        # de-init ports
        port_init_handle(handle, port, is_init=False)
        # un-bind ports
        port_swid_bind_set(handle, port, un_bind_swid)
        # disable mapping
        port_unmapping_set(handle, port)

    # Change port mapping
    for i, (port, module, lane_bmap) in enumerate(zip(new_ports_list, args.new_modules, args.new_lane_bmaps)):
        port_mapping_attr_set(handle, port, module, lane_bmap)

    # bind ports
    for port in new_ports_list:
        swid = SWID
        port_swid_bind_set(handle, port, swid)

    # init ports
    for port in new_ports_list:
        port_init_handle(handle, port, is_init=True)

    # init ports
    if chip_type == SX_CHIP_TYPE_SPECTRUM:
        for port in new_ports_list:
            # set port speed - use legacy APIs, 10G
            port_admin_speed_10G_set(handle, port)
    else:
        for port in new_ports_list:
            # set port speed - use new APIs, default 1G
            port_rate_configuration(handle, port, admin_rate)

    # configure RSTP forwarding state
    for port in new_ports_list:
        rstp_port_state_set(handle, port, SX_MSTP_INST_PORT_STATE_FORWARDING)

    # add configured ports into the specified below VLAN
    for port in new_ports_list:
        vlan = 1
        add_ports_to_vlan(handle, vlan, {port: SX_UNTAGGED_MEMBER})

    # bring port into the UP state
    for port in new_ports_list:
        port_state_set(handle, port, SX_PORT_ADMIN_STATUS_UP)


def ports_mapping_info(handle, ports_list):
    for port in ports_list:
        module_port, width, lane_bmap = ports_mapping_attr_get(handle, port)
        print ("log_port:0x%x width: %d lane_bmap: 0x%x module_port: %d" % (port, width, lane_bmap, module_port))

######################################################
#    Port split example
######################################################
def port_split_example():
    new_ports = args.new_ports
    old_ports = args.old_ports

    print "[+] opening sdk"
    rc, handle = sx_api_open(None)
    print ("sx_api_open handle:0x%x , rc %d " % (handle, rc))
    if (rc != SX_STATUS_SUCCESS):
        print "Failed to open api handle.\nPlease check that SDK is running."
        sys.exit(errno.EACCES)

    rc = sxd_access_reg_init(0, None, 4)
    if (rc != SXD_STATUS_SUCCESS):
        print ("Failed to initializing register access.\nPlease check that SDK is running.")
        sys.exit(errno.EACCES)

    _chip_type = get_chip_type()

    # Reset port speed API type flag
    reset_port_type_api_flag(handle, new_ports)

    # Print current port mapping attributes
    ports_mapping_info(handle, old_ports)

    # Split or unsplit
    split_unsplit_ports(handle, _chip_type, old_ports, new_ports)

    # Print new port mapping attributes
    ports_mapping_info(handle, new_ports)

    print("Success.")

    sx_api_close(handle)


################################################################################
#                             Main                                             #
################################################################################
if __name__ == "__main__":
    port_split_example()
