#!/usr/bin/env python
"""
This file contains a python commands example for the SDK.
Python commands syntax is very similar to the SwitchX SDK APIs.
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.
"""

import sys
import errno
from python_sdk_api.sx_api import *
from python_sdk_api.sxd_api import *
from test_infra_common import *
import argparse

######################################################
#    defines
######################################################
SWID = 0
DEVICE_ID = 1
MAX_PORTS = 64


def auto_int(x):
    return int(x, 0)


ERR_FILE_LOCATION = '/tmp/python_err_log.txt'

if len(sys.argv) < 5:
    print "Bad parameters."
    print "    " + sys.argv[0] + " --log_port 0x10100 --ingress_filter_mode <disable|enable> "
    sys.exit(0)

parser = argparse.ArgumentParser(description='sx_api_vlan_port_ingr_filter_set API')
parser.add_argument('--log_port', type=auto_int, default=0, help='Log port ')
parser.add_argument('--ingress_filter_mode', type=str, choices=["ENABLE", "DISABLE", "enable", "disable", "0", "1"],
                    help='ingress_filter_mode')
args = parser.parse_args()

ingress_filter_mode_dict = {
    "ENABLE": SX_INGR_FILTER_ENABLE,
    "DISABLE": SX_INGR_FILTER_DISABLE,
    "enable": SX_INGR_FILTER_ENABLE,
    "disable": SX_INGR_FILTER_DISABLE,
    "1": SX_INGR_FILTER_ENABLE,
    "0": SX_INGR_FILTER_DISABLE,
}

print "log_port: 0x%x" % args.log_port
print "ingress_filter_mode: %d" % ingress_filter_mode_dict[args.ingress_filter_mode]

# file_exist = os.path.isfile(ERR_FILE_LOCATION)
# sys.stderr = open(ERR_FILE_LOCATION, 'w')
# if not file_exist:
#     os.chmod(ERR_FILE_LOCATION, 0o777)

######################################################
#    functions
######################################################


def main():
    #     pdb.set_trace()
    print "[+] opening sdk"
    rc, handle = sx_api_open(None)
    print ("sx_api_open handle:0x%x , rc %d " % (handle, rc))
    if (rc != SX_STATUS_SUCCESS):
        print "Failed to open api handle.\nPlease check that SDK is running."
        sys.exit(errno.EACCES)

    rc = sx_api_vlan_port_ingr_filter_set(handle, args.log_port, ingress_filter_mode_dict[args.ingress_filter_mode])
    assert rc == SX_STATUS_SUCCESS, "sx_api_vlan_ingr_filter_set failed. [%s(%d)]" % (sx_status_dict[rc], rc)

    ingress_filter_mode_p = new_sx_ingr_filter_mode_t_p()

    rc = sx_api_vlan_port_ingr_filter_get(handle, args.log_port, ingress_filter_mode_p)
    assert rc == SX_STATUS_SUCCESS, "sx_api_vlan_ingr_filter_set failed. [%s(%d)]" % (sx_status_dict[rc], rc)

    ingr_filter_mode = sx_ingr_filter_mode_t_p_value(ingress_filter_mode_p)
    print "GET results"
    print "    Log port:0x%x " % args.log_port
    print "    ingress_filter_mode:%d " % ingr_filter_mode

    print "[+] Closing sdk"
    rc = sx_api_close(handle)


################################################################################
#                             Main                                             #
################################################################################
if __name__ == "__main__":
    main()
