#!/usr/bin/env python
'''
Python commands syntax is very similar to the Switch SDK APIs.
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

This utility set the power mode a given module.
'''
import sys
import errno
import argparse
from python_sdk_api.sx_api import *
from test_infra_common import *
import python_sdk_api.sx_api as sx_api

SPECTRUM_SWID = 0
######################################################
#  power mode and reset utility functions.
######################################################


def mgmt_phy_mod_pwr_attr_set(handle, module_id, power_attr_type, admin_pwr_mode):
    sx_mgmt_phy_mod_pwr_attr = sx_mgmt_phy_mod_pwr_attr_t()
    sx_mgmt_phy_mod_pwr_mode_attr = sx_mgmt_phy_mod_pwr_mode_attr_t()
    sx_mgmt_phy_mod_pwr_attr.power_attr_type = power_attr_type
    sx_mgmt_phy_mod_pwr_mode_attr.admin_pwr_mode_e = admin_pwr_mode
    sx_mgmt_phy_mod_pwr_attr.pwr_mode_attr = sx_mgmt_phy_mod_pwr_mode_attr
    sx_mgmt_phy_mod_pwr_attr_p = new_sx_mgmt_phy_mod_pwr_attr_t_p()
    sx_mgmt_phy_mod_pwr_attr_t_p_assign(sx_mgmt_phy_mod_pwr_attr_p, sx_mgmt_phy_mod_pwr_attr)
    try:
        rc = sx_mgmt_phy_mod_pwr_attr_set(handle, SX_ACCESS_CMD_SET, module_id, sx_mgmt_phy_mod_pwr_attr_p)
        assert SX_STATUS_SUCCESS == rc, "sx_mgmt_phy_mod_pwr_attr_set failed"
    finally:
        delete_sx_mgmt_phy_mod_pwr_attr_t_p(sx_mgmt_phy_mod_pwr_attr_p)


def mgmt_phy_mod_pwr_attr_get(handle, module_id, power_attr_type):
    sx_mgmt_phy_mod_pwr_attr_p = new_sx_mgmt_phy_mod_pwr_attr_t_p()
    sx_mgmt_phy_mod_pwr_attr = sx_mgmt_phy_mod_pwr_attr_t()
    sx_mgmt_phy_mod_pwr_attr.power_attr_type = power_attr_type
    sx_mgmt_phy_mod_pwr_attr_t_p_assign(sx_mgmt_phy_mod_pwr_attr_p, sx_mgmt_phy_mod_pwr_attr)
    try:
        rc = sx_mgmt_phy_mod_pwr_attr_get(handle, module_id, sx_mgmt_phy_mod_pwr_attr_p)
        assert SX_STATUS_SUCCESS == rc, "sx_mgmt_phy_mod_pwr_attr_get failed"
        sx_mgmt_phy_mod_pwr_attr = sx_mgmt_phy_mod_pwr_attr_t_p_value(sx_mgmt_phy_mod_pwr_attr_p)
        pwr_mode_attr = sx_mgmt_phy_mod_pwr_attr.pwr_mode_attr
        return pwr_mode_attr.admin_pwr_mode_e, pwr_mode_attr.oper_pwr_mode_e
    finally:
        delete_sx_mgmt_phy_mod_pwr_attr_t_p(sx_mgmt_phy_mod_pwr_attr_p)


def pwr_attr_set(module_id, ports, attr_type, power_mode):
    # bring the port down
    for port in ports:
        port_state_set(handle, port, SX_PORT_ADMIN_STATUS_DOWN)
    # set the desired power mode
    mgmt_phy_mod_pwr_attr_set(handle, module_id, attr_type, power_mode)
    # get the current port power mode and make sure that it is as per the mode set.
    admin_pwr_mode, oper_pwr_mode = mgmt_phy_mod_pwr_attr_get(handle, module_id, attr_type)
    assert power_mode == admin_pwr_mode, "power mode mismatch"
    # bring the port up
    port_state_set(handle, port, SX_PORT_ADMIN_STATUS_UP)

def run_example(handle, cmd, module_id):
    # construct the port module map.
    ports_attributes_list = ports_attributes_get(handle)
    port_module_map, module_port_map = get_port_module_mapping(ports_attributes_list, len(ports_attributes_list))

    if module_id not in module_port_map.keys():
        print("Error: Invalid Module Id")
        sys.exit(0)

    if cmd == "enable":
        pwr_attr_set(module_id, module_port_map[module_id], SX_MGMT_PHY_MOD_PWR_ATTR_PWR_MODE_E, SX_MGMT_PHY_MOD_PWR_MODE_LOW_E)
        print "Enabled low power mode for module [%d]" % (module_id)
    elif cmd == "disable":
        pwr_attr_set(module_id, module_port_map[module_id], SX_MGMT_PHY_MOD_PWR_ATTR_PWR_MODE_E, SX_MGMT_PHY_MOD_PWR_MODE_AUTO_E)
        print "Disabled low power mode for module [%d]" % (module_id)
    else:
        print("Error: Invalid command")
        sys.exit(0)
        


def parse_example_attributes():

    parser = argparse.ArgumentParser(description='Module power mode set utility')
    parser.add_argument('--cmd', type=str, help="enable, disable", required=True)
    parser.add_argument('--module_id', type=int, help="Moduleid", required=True)
    args = parser.parse_args()

    if len(sys.argv) < 3:
        print "Usage:"
        print "Example: Set Low power mode enable  sx_mgmt_module_low_power_mode_set.py --cmd enable --module_id 1"
        print "Example: Set Low power mode disable sx_mgmt_module_low_power_mode_set.py --cmd disable --module_id 1"
        sys.exit(0)

    module_id = args.module_id
    cmd = args.cmd

    return cmd, module_id


################################################
# Run the MAIN function
################################################
if __name__ == "__main__":

    cmd, module_id = parse_example_attributes()
    print "[+] opening sdk"
    rc, handle = sx_api_open(None)
    print ("sx_api_open handle:0x%x , rc %d " % (handle, rc))

    if (rc != SX_STATUS_SUCCESS):
        print "Failed to open api handle.\nPlease check that SDK is running."
        sys.exit(errno.EACCES)

    # Run the  example with retrieved above parameters
    run_example(handle, cmd, module_id)

    sx_api_close(handle)
