#!/usr/bin/env python

import sys
import errno
import time
import argparse
from python_sdk_api.sxd_api import *


def parse_arguments():
    parser = argparse.ArgumentParser("Example for getting switch fans RPM & Tempartures")
    parser.add_argument("-n", "--no_timeout",
                        dest="no_timeout",
                        action="store_true",
                        help="Run example until force stop (ctrl+c)")
    return parser.parse_args()


args = parse_arguments()

MIN_IDX = 2
MAX_IDX = 8

print "[+] initializing register access"
rc = sxd_access_reg_init(0, None, 4)
if (rc != 0):
    print "Failed to initializing register access.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

print "[+] reading fan speed"
mfsm = ku_mfsm_reg()
mfsm.tacho = 9

meta = sxd_reg_meta_t()
meta.dev_id = 1
meta.swid = 0
meta.access_cmd = SXD_ACCESS_CMD_GET

names = "\t".join(["Fan #{0}".format(x) for x in range(MIN_IDX, MAX_IDX + 1)])
names += "\tTemp"
print names

fans = [ku_mfsm_reg() for _ in range(MIN_IDX, MAX_IDX + 1)]
for x in range(MAX_IDX - MIN_IDX + 1):
    fans[x].tacho = MIN_IDX + x


mtmp = ku_mtmp_reg()
sxd_access_reg_mtmp(mtmp, meta, 1, None, None)

while True:
    rpms = []
    for fan in fans:
        sxd_access_reg_mfsm(fan, meta, 1, None, None)
        rpms.append(str(fan.rpm))
    sxd_access_reg_mtmp(mtmp, meta, 1, None, None)
    rpms.append(str(mtmp.temperature * 0.125) + " C")
    sys.stdout.write("\t".join(rpms) + "\r")
    sys.stdout.flush()
    if not args.no_timeout:
        sys.stdout.write("\n")
        sys.exit(0)
    time.sleep(1)
