#!/usr/bin/env python

import sys
import time
import os
from python_sdk_api.sxd_api import *
import argparse

parser = argparse.ArgumentParser(description='get / set MFGD',
                                 formatter_class=argparse.ArgumentDefaultsHelpFormatter)
parser.add_argument('--device_id', default=1, type=lambda x: int(x, 0), help='The device id to set the FW MFGD on')
args = parser.parse_args()


def main():

    print "[+] MFGD register access Example Test start"
    print "[+] initializing register access"
    sxd_access_reg_init(0, None, 4)

    mfgd = ku_mfgd_reg()
    orig_mfgd = ku_mfgd_reg()

    meta = sxd_reg_meta_t()
    meta.dev_id = args.device_id
    meta.swid = 0

#     import pdb
#     pdb.set_trace()

    orig_mfgd.fw_dci_rif_cache = 0
    orig_mfgd.fw_dci_en = 0
    orig_mfgd.fw_kvc_en = 0
    orig_mfgd.tcr_dbg_en = 0
    orig_mfgd.trigger_stack_overflow = 0
    orig_mfgd.trigger_test = 0
    orig_mfgd.trigger_test_storm = 0
    orig_mfgd.fw_fatal_mode = 0
    orig_mfgd.atcam_bf_en = 0
    orig_mfgd.egress_en = 0
    orig_mfgd.fw_init_total_time = 0
    orig_mfgd.fw_init_open_ports_time = 0

    print "\n========================"
    print "[+] GET test MFGD content"
    print "========================"
    meta.access_cmd = SXD_ACCESS_CMD_GET
    print "[+] performing get MFGD"
    rc = sxd_access_reg_mfgd(orig_mfgd, meta, 1, None, None)
    assert rc == 0, "Failed to read MFGD register, rc: %d" % (rc)

    print "[+] MFGD content:"
    print "------------------------"
    print "[+] fw_dci_rif_cache:", orig_mfgd.fw_dci_rif_cache
    print "[+] fw_dci_en:", orig_mfgd.fw_dci_en
    print "[+] fw_kvc_en:", orig_mfgd.fw_kvc_en
    print "[+] tcr_dbg_en:", orig_mfgd.tcr_dbg_en
    print "[+] trigger_stack_overflow:", orig_mfgd.trigger_stack_overflow
    print "[+] trigger_test:", orig_mfgd.trigger_test
    print "[+] trigger_test_storm:", orig_mfgd.trigger_test_storm
    print "[**] fw_fatal_mode:", orig_mfgd.fw_fatal_mode
    print "[+] atcam_bf_en:", orig_mfgd.atcam_bf_en
    print "[+] egress_en:", orig_mfgd.egress_en
    print "[+] fw_init_total_time:", orig_mfgd.fw_init_total_time
    print "[+] fw_init_open_ports_time:", orig_mfgd.fw_init_open_ports_time

    print "\n========================="
    print "[+] SET test MFGD content"
    print "=========================="
    # get defaults first to prevent issues
    rc = sxd_access_reg_mfgd(mfgd, meta, 1, None, None)
    assert rc == 0, "Failed to read MFGD register, rc: %d" % (rc)

    set_value = SXD_MFGD_MODE_FW_FATAL_N_CONTINUE_E
    meta.access_cmd = SXD_ACCESS_CMD_SET
    mfgd.fw_fatal_mode = set_value
    mfgd.trigger_test = 0
    print "--------------------------------"
    print "[+] performing set MFGD content:"
    print "[+] **fw_fatal_mode:", mfgd.fw_fatal_mode
    print "[+] set MFGD "
    sxd_access_reg_mfgd(mfgd, meta, 1, None, None)

    meta.access_cmd = SXD_ACCESS_CMD_GET
    print "[+] set MFGD (verify)"
    sxd_access_reg_mfgd(mfgd, meta, 1, None, None)

    print "[+] get after Set MFGD content:"
    print "--------------------------------"
    print "[+] fw_dci_rif_cache:", orig_mfgd.fw_dci_rif_cache
    print "[+] fw_dci_en:", orig_mfgd.fw_dci_en
    print "[+] fw_kvc_en:", orig_mfgd.fw_kvc_en
    print "[+] tcr_dbg_en:", orig_mfgd.tcr_dbg_en
    print "[+] trigger_stack_overflow:", orig_mfgd.trigger_stack_overflow
    print "[+] trigger_test:", orig_mfgd.trigger_test
    print "[+] trigger_test_storm:", orig_mfgd.trigger_test_storm
    print "[**] fw_fatal_mode:", orig_mfgd.fw_fatal_mode
    print "[+] atcam_bf_en:", orig_mfgd.atcam_bf_en
    print "[+] egress_en:", orig_mfgd.egress_en
    print "[+] fw_init_total_time:", orig_mfgd.fw_init_total_time
    print "[+] fw_init_open_ports_time:", orig_mfgd.fw_init_open_ports_time

    assert mfgd.fw_fatal_mode == set_value, "MFGD mode after set is not:[%d]" % (set_value)

    print "\n============================="
    print "[+] use MFGD to trigger MFDE:"
    print "=============================="
    set_value = SXD_MFGD_MODE_FW_FATAL_N_CONTINUE_E
    meta.access_cmd = SXD_ACCESS_CMD_SET
    meta.mode = SXD_ACCESS_MODE_ASYNC
    mfgd.fw_fatal_mode = set_value
    mfgd.trigger_test = 1
    sxd_access_reg_mfgd(mfgd, meta, 1, None, None)

    print "\n============================="
    print "[+] set MFGD to original value:"
    print "=============================="
    meta.access_cmd = SXD_ACCESS_CMD_SET
    meta.mode = SXD_ACCESS_MODE_ASYNC
    sxd_access_reg_mfgd(orig_mfgd, meta, 1, None, None)
    print "\n\n========================================="
    print "[+] MFGD register access example test end rc: %d" % (rc)
    print "=========================================\n\n"


################################################################################
#                             Main                                             #
################################################################################
if __name__ == '__main__':
    main()
