#!/usr/bin/env python
# -*- python -*-

#######################################################################
# global params
#######################################################################
PORT_TYPE_NVE = 8
PORT_TYPE_CPU = 4
PORT_TYPE_VPORT = 2
PORT_TYPE_OFFSET = 28
PORT_TYPE_MASK = 0xF0000000
PORT_OPER_STATUS = 0
PORT_ADMIN_STATUS = 1
MAX_COUNT_SLEEPS = 100
SPECTRUM_SWID = 0
#######################################################################
# functions
#######################################################################
from python_sdk_api.sx_api import *
import sys
import os
import socket
import struct
import time

sx_status_dict = {
    SX_STATUS_SUCCESS: "Success",
    SX_STATUS_ERROR: "Internal Error",
    SX_STATUS_SDK_NOT_INITIALIZED: "SwitchX SDK wasn't Initialized",
    SX_STATUS_INVALID_HANDLE: "Invalid Handle",
    SX_STATUS_COMM_ERROR: "Communication Error",
    SX_STATUS_NO_RESOURCES: "No More Resources",
    SX_STATUS_NO_MEMORY: "No More Memory",
    SX_STATUS_MEMORY_ERROR: "Memory Error",
    SX_STATUS_CMD_ERROR: "Command Error",
    SX_STATUS_CMD_INCOMPLETE: "Command Not Completed",
    SX_STATUS_CMD_UNSUPPORTED: "Command Unsupported",
    SX_STATUS_CMD_UNPERMITTED: "Command Unpermitted",
    SX_STATUS_PARAM_NULL: "Parameter NULL",
    SX_STATUS_PARAM_ERROR: "Parameter Error",
    SX_STATUS_PARAM_EXCEEDS_RANGE: "Parameter Exceeds Range",
    SX_STATUS_MESSAGE_SIZE_ZERO: "Message Size Exceeds Limit",
    SX_STATUS_MESSAGE_SIZE_EXCEEDS_LIMIT: "Message Size Exceeds Limit",
    SX_STATUS_DB_ALREADY_INITIALIZED: "Database Already Initialized",
    SX_STATUS_DB_NOT_INITIALIZED: "Database Wasn't Initialized",
    SX_STATUS_DB_NOT_EMPTY: "Database Not Empty",
    SX_STATUS_END_OF_DB: "End of Database",
    SX_STATUS_ENTRY_NOT_FOUND: "Entry Not Found",
    SX_STATUS_ENTRY_ALREADY_EXISTS: "Entry Already Exists",
    SX_STATUS_ENTRY_NOT_BOUND: "Entry Not Bound",
    SX_STATUS_ENTRY_ALREADY_BOUND: "Entry Already bound",
    SX_STATUS_WRONG_POLICER_TYPE: "Wrong Policer Type",
    SX_STATUS_UNEXPECTED_EVENT_TYPE: "Unexpected event type",
    SX_STATUS_TRAP_ID_NOT_CONFIGURED: "Trap ID not configured",
    SX_STATUS_INT_COMM_CLOSE: "Internal Communication close",
    SX_STATUS_RESOURCE_IN_USE: "Resource is in use",
    SX_STATUS_EVENT_TRAP_ALREADY_ASSOCIATED: "Trap ID is already associated to another trap priority",
    SX_STATUS_ALREADY_INITIALIZED: "Already initialized",
    SX_STATUS_TIMEOUT: "TIMEOUT",
    SX_STATUS_MODULE_UNINITIALIZED: "Module is uninitialized",
    SX_STATUS_UNSUPPORTED: "Operation Unsupported",
    SX_STATUS_SX_UTILS_RETURNED_NON_ZERO: "Utils return status is Non-Zero",
    SX_STATUS_PARTIALLY_COMPLETE: "Partially complete",
    SX_STATUS_ACCEPTED: "Accepted",
    SX_STATUS_ISSU_IN_PROGRESS: "ISSU in progress",
}

vxlan_type_dict = {
    8: 'NVE VXLAN',
    12: 'NVE VXLAN IPv6'}


map_dir_dict = {0: 'BIDIR',
                1: 'DECAP'}


direction_dict = {0: 'NONE',
                  1: 'ENCAP',
                  2: 'DECAP',
                  3: 'SYMMETRIC'
                  }

underlay_domain_type_dict = {0: 'VRID',
                             1: 'RIF'}

ttl_cmd_dict = {1: 'SET',
                2: 'COPY',
                3: 'PRESERVE',
                4: 'MINIMUM'}

hash_cmd_dict = {1: 'ZERO',
                 2: 'CALCULATE'}

dscp_rewrite_action_dict = {0: 'PRESERVE',
                            2: 'DISABLE',
                            3: 'ENABLE'}

tunnel_cost_dscp_action_dict = {0: 'PRESERVE',
                                1: 'COPY',
                                2: 'SET'}

dscp_action_dict = {0: 'PRESERVE',
                    1: 'COPY',
                    2: 'SET'}

ecn_map_dict = {0: 'Non ECT(00)',
                1: 'ECT1(01)',
                2: 'ECT0(10)',
                3: 'CE(11)'}
tunnel_type_str_dict = \
    {
        SX_TUNNEL_TYPE_NVE_VXLAN: "NVE_VXLAN",
        SX_TUNNEL_TYPE_NVE_VXLAN_GPE: "NVE_VXLAN_GPE",
        SX_TUNNEL_TYPE_NVE_GENEVE: "NVE_GENEVE",
        SX_TUNNEL_TYPE_NVE_NVGRE: "NVE_NVGRE",
        SX_TUNNEL_TYPE_NVE_VXLAN_IPV6: "NVE_VXLAN_IPV6",
        SX_TUNNEL_TYPE_NVE_NVGRE_IPV6: "NVE_NVGRE_IPV6",
        SX_TUNNEL_TYPE_IPINIP_P2P_IPV4_IN_IPV4: "IPV4_IN_IPV4",
        SX_TUNNEL_TYPE_IPINIP_P2P_IPV4_IN_GRE: "IPV4_IN_GRE",
        SX_TUNNEL_TYPE_IPINIP_P2P_IPV6_IN_IPV4: "IPV6_IN_IPV4",
        SX_TUNNEL_TYPE_IPINIP_P2P_IPV6_IN_IPV4_WITH_GRE: "IPV6_IN_IPV4_GRE",
        SX_TUNNEL_TYPE_IPINIP_P2P_IPV4_IN_IPV6: "IPV4_IN_IPV6",
        SX_TUNNEL_TYPE_IPINIP_P2P_IPV4_IN_IPV6_WITH_GRE: "IPV4_IN_IPV6_GRE",
        SX_TUNNEL_TYPE_IPINIP_P2P_IPV6_IN_IPV6: "IPV6_IN_IPV6",
        SX_TUNNEL_TYPE_IPINIP_P2P_IPV6_IN_GRE: "IPV6_IN_GRE",
    }

trap_attr_prio_str_dict = \
    {
        SX_TRAP_PRIORITY_BEST_EFFORT: "Best Effort",
        SX_TRAP_PRIORITY_LOW: "Low",
        SX_TRAP_PRIORITY_MED: "Med",
        SX_TRAP_PRIORITY_HIGH: "High",
        SX_TRAP_PRIORITY_CRITICAL: "Critical"
    }

router_action_str_dict =\
    {
        SX_ROUTER_ACTION_DROP: "Drop",
        SX_ROUTER_ACTION_TRAP: "Trap",
        SX_ROUTER_ACTION_FORWARD: "Forward",
        SX_ROUTER_ACTION_MIRROR: "Mirror (deprecated)",
        SX_ROUTER_ACTION_TRAP_FORWARD: "Trap and Forward",
        SX_ROUTER_ACTION_SPAN: "Span",
    }

tnl_decap_rule_key_type_str_dict =\
    {
        SX_TUNNEL_DECAP_KEY_FIELDS_TYPE_DIP: "DIP",
        SX_TUNNEL_DECAP_KEY_FIELDS_TYPE_DIP_SIP: "DIP_SIP"
    }


def check_vport(port):
    port_type = (port & PORT_TYPE_MASK) >> PORT_TYPE_OFFSET
    return port_type & PORT_TYPE_VPORT


def check_nve(port):
    port_type = (port & PORT_TYPE_MASK) >> PORT_TYPE_OFFSET
    return port_type & PORT_TYPE_NVE


def check_cpu(port):
    port_type = (port & PORT_TYPE_MASK) >> PORT_TYPE_OFFSET
    return port_type & PORT_TYPE_CPU


def get_ports(handle, ports_num):
    port_cnt_p = new_uint32_t_p()
    port_list = []
    log_port_list_p = new_sx_port_log_id_t_arr(ports_num)
    uint32_t_p_assign(port_cnt_p, ports_num)
    rc = sx_api_port_swid_port_list_get(handle, 0, log_port_list_p, port_cnt_p)
    if rc != 0:
        raise Exception('Failed to get {} ports.'
                        'sx_api_port_swid_port_list_get returned rc: {}'.format(ports_num, rc))

    for i in xrange(ports_num):
        port_list.append(sx_port_log_id_t_arr_getitem(log_port_list_p, i))
    return port_list


def mapPortAndInterfaces(handle, num_ports=4):

    count_sleeps = 0

    port_with_admin_up_list = getAllPortWithLinkUp(handle, PORT_ADMIN_STATUS)

    if num_ports > len(port_with_admin_up_list):
        raise Exception("Not enough ports in state up")

    print '[{}]'.format(', '.join(hex(x) for x in port_with_admin_up_list))

    return port_with_admin_up_list


def wait_till_port_list_up(handle, interface_list, timeout=50):
    for index in range(4):
        wait_till_port_up(handle, interface_list[index].get_log_port(), timeout)
    return True


def getPortDown(handle, port_list):
    for port in port_list:
        if checkPortDown(handle, port):
            return port
    raise Exception("no port in state down")


def wait_till_port_up(handle, port, timeout=50):
    start = time.time()
    end = time.time()

    while not checkPortUp(handle, port):
        end = time.time()
        if timeout < end - start:
            print (" \n", end - start)
            sys.exit(1)

    print (" \n", end - start)

    return True


def checkPortUp(handle, port):

    oper_state_t_p = new_sx_port_oper_state_t_p()
    admin_state_t_p = new_sx_port_admin_state_t_p()
    module_state_t_p = new_sx_port_module_state_t_p()

    rc = sx_api_port_state_get(handle, port, oper_state_t_p, admin_state_t_p, module_state_t_p)
    if rc != 0:
        sys.exit(1)

    oper_state = sx_port_oper_state_t_p_value(oper_state_t_p)
    return (oper_state == SX_PORT_OPER_STATUS_UP)


def checkPortDown(handle, port):

    oper_state_t_p = new_sx_port_oper_state_t_p()
    admin_state_t_p = new_sx_port_admin_state_t_p()
    module_state_t_p = new_sx_port_module_state_t_p()

    rc = sx_api_port_state_get(handle, port, oper_state_t_p, admin_state_t_p, module_state_t_p)
    if rc != 0:
        sys.exit(1)

    oper_state = sx_port_oper_state_t_p_value(oper_state_t_p)
    return (oper_state == SX_PORT_OPER_STATUS_DOWN)


def getAllPortWithLinkUp(handle, port_mode=PORT_OPER_STATUS):

    port_with_link_up_list = []

    # Get the active SWID list

    swid_list_p = new_sx_swid_t_arr(16)

    swid_cnt_p = new_uint32_t_p()

    uint32_t_p_assign(swid_cnt_p, 16)

    rc = sx_api_port_swid_list_get(handle, swid_list_p, swid_cnt_p)
    if rc != 0:
        sys.exit(1)

    swid_cnt = uint32_t_p_value(swid_cnt_p)

    #print ("sx_api_port_swid_list_get swid_cnt:%d , rc %d, swid table: " % ( swid_cnt, rc) )

    for i in range(0, swid_cnt):

        swid = sx_swid_t_arr_getitem(swid_list_p, i)

        #print ("[%d] swid %d" % ( i, swid) )

        log_port_list_p = new_sx_port_log_id_t_arr(64)

        port_cnt_p = new_uint32_t_p()

        uint32_t_p_assign(port_cnt_p, 64)

        rc = sx_api_port_swid_port_list_get(handle, swid, log_port_list_p, port_cnt_p)
        if rc != 0:
            sys.exit(1)

        port_cnt = uint32_t_p_value(port_cnt_p)

        #print ("sx_api_port_swid_port_list_get port_cnt:%d , rc %d, port table: " % ( port_cnt, rc) )

        for i in range(0, port_cnt):

            port = sx_port_log_id_t_arr_getitem(log_port_list_p, i)

            #print ("[%d] port 0x%x" % ( i, port) )

            oper_state_t_p = new_sx_port_oper_state_t_p()
            admin_state_t_p = new_sx_port_admin_state_t_p()
            module_state_t_p = new_sx_port_module_state_t_p()

            rc = sx_api_port_state_get(handle, port, oper_state_t_p, admin_state_t_p, module_state_t_p)
            if rc != 0:
                sys.exit(1)

            oper_state = sx_port_oper_state_t_p_value(oper_state_t_p)
            admin_state = sx_port_admin_state_t_p_value(admin_state_t_p)
            module_state = sx_port_module_state_t_p_value(module_state_t_p)

            if port_mode == PORT_OPER_STATUS:
                if oper_state == SX_PORT_OPER_STATUS_UP:
                    # print "port state is up"
                    port_with_link_up_list.append(port)
            elif port_mode == PORT_ADMIN_STATUS:
                if admin_state == SX_PORT_ADMIN_STATUS_UP:
                    port_with_link_up_list.append(port)

    return port_with_link_up_list


def port_state_set(handle, log_port, admin_state):
    rc = sx_api_port_state_set(handle, log_port, admin_state)
    assert SX_STATUS_SUCCESS == rc, "sx_api_port_state_set failed"


def get_port_module_mapping(port_attributes_list, port_cnt):
    port_module = {}
    module_port = dict()
    for i in range(port_cnt):
        port = port_attributes_list[i]
        # consider all external/network ports
        if port.port_mode == SX_PORT_MODE_EXTERNAL:
            # populate {log_port: module_id} mapping
            port_module[hex(port.log_port)] = port.port_mapping.module_port
            # populate {module_id: log_port_list} mapping
            if port.port_mapping.module_port not in module_port.keys():
                module_port[port.port_mapping.module_port] = [port.log_port]
            else:
                module_port[port.port_mapping.module_port].append(port.log_port)
    # return the dictionary
    return port_module, module_port


def ports_attributes_get(handle):
    port_attributes_list = None
    port_cnt_p = new_uint32_t_p()
    try:
        rc = sx_api_port_device_get(handle, 1, SPECTRUM_SWID, None, port_cnt_p)
        assert SX_STATUS_SUCCESS == rc, "sx_api_port_device_get failed to get ports count"
        num_ports = uint32_t_p_value(port_cnt_p)

        port_attributes_list = new_sx_port_attributes_t_arr(num_ports)
        rc = sx_api_port_device_get(handle, 1, SPECTRUM_SWID, port_attributes_list, port_cnt_p)
        assert SX_STATUS_SUCCESS == rc, "sx_api_port_device_get failed to get ports count"

        return [sx_port_attributes_t_arr_getitem(port_attributes_list, i) for i in xrange(num_ports)]
    finally:
        delete_uint32_t_p(port_cnt_p)
        if port_attributes_list is not None:
            delete_sx_port_attributes_t_arr(port_attributes_list)


def ip_addr_to_str(ip_addr):
    if ip_addr.version == SX_IP_VERSION_IPV4:
        return socket.inet_ntop(socket.AF_INET, struct.pack('!I', ip_addr.addr.ipv4.s_addr))
    else:
        bytes = ip_addr.addr.ipv6.__in6_u.__u6_addr8
        byte_arr = []
        for i in xrange(0, 16):
            byte_arr.append(uint8_t_arr_getitem(bytes, i))
        if sys.byteorder == "little":
            dwords = struct.pack("!BBBBBBBBBBBBBBBB", byte_arr[3], byte_arr[2], byte_arr[1], byte_arr[0], byte_arr[7], byte_arr[6], byte_arr[5], byte_arr[4], byte_arr[11], byte_arr[10], byte_arr[9], byte_arr[8], byte_arr[15], byte_arr[14], byte_arr[13], byte_arr[12])
        else:
            dwords = struct.pack("!BBBBBBBBBBBBBBBB", byte_arr[0], byte_arr[1], byte_arr[2], byte_arr[3], byte_arr[4], byte_arr[5], byte_arr[6], byte_arr[7], byte_arr[8], byte_arr[9], byte_arr[10], byte_arr[11], byte_arr[12], byte_arr[13], byte_arr[14], byte_arr[15])
        return socket.inet_ntop(socket.AF_INET6, dwords)


def redirect_stdout():
    sys.stdout.flush()
    stdout_bak = os.dup(1)
    devnull = os.open('/dev/null', os.O_WRONLY)
    os.dup2(devnull, 1)
    os.close(devnull)
    return stdout_bak


def print_all_attr_from_a_struct(struct):
    for attr in dir(struct):
        if not attr.startswith("__") and not attr == "this":
            print "%s = %s" % (attr, getattr(struct, attr))


def ip_prefix_to_str(ip_prefix):
    if ip_prefix.version == SX_IP_VERSION_IPV4:
        return socket.inet_ntop(socket.AF_INET, struct.pack('!I', ip_prefix.prefix.ipv4.addr.s_addr))
    else:
        bytes = ip_prefix.prefix.ipv6.addr.__in6_u.__u6_addr8
        byte_arr = []
        for i in xrange(0, 16):
            byte_arr.append(uint8_t_arr_getitem(bytes, i))
        if sys.byteorder == "little":
            dwords = struct.pack("!BBBBBBBBBBBBBBBB", byte_arr[3], byte_arr[2], byte_arr[1], byte_arr[0], byte_arr[7], byte_arr[6], byte_arr[5], byte_arr[4], byte_arr[11], byte_arr[10], byte_arr[9], byte_arr[8], byte_arr[15], byte_arr[14], byte_arr[13], byte_arr[12])
        else:
            dwords = struct.pack("!BBBBBBBBBBBBBBBB", byte_arr[0], byte_arr[1], byte_arr[2], byte_arr[3], byte_arr[4], byte_arr[5], byte_arr[6], byte_arr[7], byte_arr[8], byte_arr[9], byte_arr[10], byte_arr[11], byte_arr[12], byte_arr[13], byte_arr[14], byte_arr[15])
        return socket.inet_ntop(socket.AF_INET6, dwords)


def mask_len(ip_prefix):
    if ip_prefix.version == SX_IP_VERSION_IPV4:
        count = 0
        mask = ip_prefix.prefix.ipv4.mask.s_addr
        while mask != 0:
            if ((mask & 0x1) == 1):
                count = count + 1
            mask = mask >> 1
        return count
    else:
        bytes = ip_prefix.prefix.ipv6.mask.__in6_u.__u6_addr8
        count = 0
        for i in xrange(0, 16):
            byte = uint8_t_arr_getitem(bytes, i)
            while byte != 0:
                if ((byte & 0x1) == 1):
                    count = count + 1
                byte = byte >> 1
        return count


def _ipv6_str_to_bytes(address):
    # Load the ipv6 string into 4 dwords
    dwords = struct.unpack("!IIII", socket.inet_pton(socket.AF_INET6, address))
    # Convert dwords endian using ntohl
    total_bytes = [socket.ntohl(i) for i in dwords]
    # Finally, convert back to bytes
    out = struct.pack(">IIII", total_bytes[0], total_bytes[1], total_bytes[2], total_bytes[3])
    return [ord(i) for i in out]


def check_ipv6_type(addr):
    if ':' in addr:  # IPV6 version
        return True
    else:
        return False


def check_ipv4_type(addr):
    if '.' in addr:  # IPV4 version
        return True
    else:
        return False


def make_sx_ip_addr_v4(addr):
    " This function creates ipv4 sx_ip_addr struct with given ip address. "

    ip_addr = sx_ip_addr_t()
    ip_addr.version = SX_IP_VERSION_IPV4
    ip_addr.addr.ipv4.s_addr = struct.unpack('>I', socket.inet_pton(socket.AF_INET, addr))[0]
    return ip_addr


def make_sx_ip_addr_v6(addr):
    " This function creates ipv6 sx_ip_addr struct with given ip address. "
    if isinstance(addr, str) or isinstance(addr, unicode):
        addr = _ipv6_str_to_bytes(addr)
    ip_addr = sx_ip_addr_t()
    ip_addr.version = SX_IP_VERSION_IPV6
    for i in xrange(0, 16):
        uint8_t_arr_setitem(ip_addr.addr.ipv6.__in6_u.__u6_addr8, i, addr[i])
    return ip_addr


def make_sx_ip_addr(addr):
    if check_ipv4_type(addr):
        return make_sx_ip_addr_v4(addr)
    elif check_ipv6_type(addr):
        return make_sx_ip_addr_v6(addr)


def make_fixed_parse_header_src_node(parser_hdr_fixed):
    from_ph = sx_flex_parser_header_t()
    from_ph.parser_hdr_type = SX_FLEX_PARSER_HEADER_FIXED_E
    from_ph.hdr_data.parser_hdr_fixed = parser_hdr_fixed
    return from_ph


def make_fixed_transition_dest_node(parser_hdr_fixed, encap_level, val):
    to_ph = sx_flex_parser_transition_t()
    to_ph.next_parser_hdr.parser_hdr_type = SX_FLEX_PARSER_HEADER_FIXED_E
    to_ph.next_parser_hdr.hdr_data.parser_hdr_fixed = parser_hdr_fixed
    to_ph.encap_level = encap_level
    to_ph.transition_value = val
    return to_ph


def vxlan_get_udp_dport(handle, default_udp_port=4789):
    next_trans_cnt_p = new_uint32_t_p()
    uint32_t_p_assign(next_trans_cnt_p, 0)
    from_ph = make_fixed_parse_header_src_node(SX_FLEX_PARSER_HEADER_UDP_E)
    rc = sx_api_flex_parser_transition_get(handle, from_ph, None, next_trans_cnt_p)
    if rc != SX_STATUS_SUCCESS:
        return default_udp_port

    next_trans_cnt = uint32_t_p_value(next_trans_cnt_p)
    next_trans_p = new_sx_flex_parser_transition_t_arr(next_trans_cnt)
    rc = sx_api_flex_parser_transition_get(handle, from_ph, next_trans_p, next_trans_cnt_p)
    if rc != SX_STATUS_SUCCESS:
        raise Exception("sx_api_flex_parser_transition_get failed %s" % (rc))
    to_trans = sx_flex_parser_transition_t()
    to_trans = sx_flex_parser_transition_t_arr_getitem(next_trans_p, 0)
    return to_trans.transition_value


def print_api_example_disclaimer():
    print "DISCLAIMER"
    print "=========="
    print "The following code is an EXAMPLE whose sole purpose is to demonstrate various capabilities  of the SDK."
    print "This example represents a common configuration scenario and is in no way a complete guide to the demonstrated feature."
    print "For complete documentation please refer to the SDK API Guide."
    print "There is no guarantee that this example will run successfully on all hardware and software configurations."
    print ""
